/**
*****************************************************************************
**
**  File        : main.c
**
**  Abstract    : main function.
**
**  Functions   : main
**
*****************************************************************************
*/

/* Includes */
#include <stddef.h>
#include "stm32f10x.h"
#include "lib_pcd8544.h"

/* Private typedef */
/* Private define  */
/* Private macro */
#define ASCII_HEX(value) (value > 9)?(value + 0x41 - 10):(value + 0x30)

/* Private variables */
uint16_t ADC1Reg = 0;
//int val_akt = 0;
float ADC1Val = 0;
uint8_t temp[3] = {0,0,0};

/* Private function prototypes */
/* Private functions */
void Configuration_RCC(void);
void Configuration_GPIO(void);
void Configuration_Interrupts(void);
void Configuration_ADC(void);
/**
**===========================================================================
**
**  Abstract: main program
**
**===========================================================================
*/
int main(void)
{
	Configuration_RCC();
	Configuration_GPIO();
	Configuration_Interrupts();
	Configuration_ADC();

	//inicjalizacja i obsuga wywietlacza
	PCD8544_INIT();
	PCD8544_LCDClear();
	PCD8544_LCDWriteText("Voltage", 21, 0);
	PCD8544_LCDWrite();

	while(1)
	{
		//
		ADC1Reg = ADC_GetConversionValue(ADC1) & 0xFF0;
		ADC1Val = (ADC1Reg * 3.3 / 4096);

		//przeksztacenie wartosci ADC1Val do postaci umoliwiajcej prezentacj na wywietlaczu
		temp[0] = (uint8_t)(ADC1Val / 1.0);
		temp[1] = (uint8_t)((ADC1Val - temp[0]) / 0.1) ;
		temp[2] = (uint8_t)((ADC1Val - temp[0] - (temp[1] * 0.1))/ 0.01);

		//prezentacja napicia na pinie PC4 na wywietlaczu
		PCD8544_LCDWriteSignDouble(temp[0] + 0x30, 12, 2);
		PCD8544_LCDWriteSignDouble('.', 24, 2);
		PCD8544_LCDWriteSignDouble(temp[1] + 0x30, 36, 2);
		PCD8544_LCDWriteSignDouble(temp[2] + 0x30, 48, 2);
		PCD8544_LCDWriteSignDouble('V', 60, 2);

		//wyswietlenie wartosi heksadecymalnej zmiennej ADC1Val
		PCD8544_LCDWriteSign('0', 27, 1);
		PCD8544_LCDWriteSign('x', 33, 1);
		PCD8544_LCDWriteSign(ASCII_HEX(ADC1Reg % 0x1000 / 0x100), 39, 1);
		PCD8544_LCDWriteSign(ASCII_HEX(ADC1Reg % 0x100 / 0x10), 45, 1);
		PCD8544_LCDWriteSign(ASCII_HEX(ADC1Reg % 0x10 / 0x1), 51, 1);

		//wyswietlenie bargrafu
		PCD8544_LCDBargraph(ADC1Reg & 0xFF0,0xFF0,5);

		//wyslanie danych na wywietlacz
		PCD8544_LCDWrite();

		//wyzerowanie 8 najstarszych bitw GPIOB
		GPIOB->BSRR = 0xFF000000;
		//ustawienie 8 najstarszych bitw GPIOB
		GPIOB->BSRR = (ADC1Reg << 4);
	}
}

void Configuration_RCC(void)
{
	ErrorStatus HSEStartUpStatus;

	//Reset ustawien RCC
	RCC_DeInit();

	//Wlaczaenie HSE
	RCC_HSEConfig(RCC_HSE_ON);

	//Zaczekaj az HSE zostaeni zainicjowany
	HSEStartUpStatus = RCC_WaitForHSEStartUp();

	//jesli HSE zosta zainicjalizowany poprawnie
	if(HSEStartUpStatus == SUCCESS)
	{
		//Opoznienie dla pamieci Flash
		FLASH_SetLatency(FLASH_Latency_2);

		//Wlaczenie buffora pamieci flash
		FLASH_PrefetchBufferCmd(FLASH_PrefetchBuffer_Enable);

		//konfiguracja preskalera AHB
		//HCLK = SYSCLK
		RCC_HCLKConfig(RCC_SYSCLK_Div1);

		//konfigracja preskalera APB2
	    //PCLK2 = HCLK = 72MHz, czestotliwosz 72MHz bedzie uzyskana dopiero po pelnej inicjalizacji RCC
		RCC_PCLK2Config(RCC_HCLK_Div1);

	    //konfigracja preskalera APB1
		//PCLK1 = HCLK/2 = 36MHz, czestotliwosz 36MHz bedzie uzyskana dopiero po pelnej inicjalizacji RCC
		RCC_PCLK1Config(RCC_HCLK_Div2);

	    //konfiguracja PLLLCK
		//PLLCLK = HSE / 1 * 9 = 72 MHz
		RCC_PLLConfig(RCC_PLLSource_HSE_Div1, RCC_PLLMul_9);

		//Enable PLL
	    RCC_PLLCmd(ENABLE);

	    //odczekanie az PLL bedzie gotowy do pracy
	    while(RCC_GetFlagStatus(RCC_FLAG_PLLRDY) == RESET){}

	    //konfiguracja SW
		//wybranie PLL jako SYSCLK
		RCC_SYSCLKConfig(RCC_SYSCLKSource_PLLCLK);

	    //odczekanie az PLLCLK bedzie sygnaem zegarowym mikrokontrolera
	    while((RCC->CFGR & (uint32_t)RCC_CFGR_SWS) != (uint32_t)0x08){}
	}

	RCC_APB2PeriphClockCmd(	RCC_APB2Periph_TIM1 |
							RCC_APB2Periph_ADC1 |
							RCC_APB2Periph_GPIOB |
							RCC_APB2Periph_GPIOC, ENABLE);
}

void Configuration_GPIO(void)
{
	GPIO_InitTypeDef GPIO_InitStructure;

	//inicjalizacja pinow obslugujacych diody
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
	GPIO_InitStructure.GPIO_Pin =	GPIO_Pin_8 |
									GPIO_Pin_9 |
									GPIO_Pin_10 |
									GPIO_Pin_11 |
									GPIO_Pin_12 |
									GPIO_Pin_13 |
									GPIO_Pin_14 |
									GPIO_Pin_15;
	GPIO_Init(GPIOB, &GPIO_InitStructure);

	//inicjalizacja pinu wejscia analogowego
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AIN;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_4;
	GPIO_Init(GPIOC, &GPIO_InitStructure);

}

void Configuration_Interrupts(void)
{
	RCC_ClocksTypeDef RCC_Clocks;
	NVIC_InitTypeDef NVIC_InitStructure;

	//skonfigurowanie miejsca przechowywania tablicy wektorow przerwan
	NVIC_SetVectorTable(NVIC_VectTab_FLASH, 0x0);

	//konfiguracja podzialu priorytetow przerwan: 4 grupy, 4 podgrupy
	NVIC_PriorityGroupConfig(NVIC_PriorityGroup_2);

	//konfiguracja przerwan pochodzacych od SysTick
	NVIC_InitStructure.NVIC_IRQChannel = SysTick_IRQn;
	NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 2;
	NVIC_InitStructure.NVIC_IRQChannelSubPriority = 0;
	NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
	NVIC_Init(&NVIC_InitStructure);

	//uzyskanie czestotliwosci poszczegolnych linii zegara
	RCC_GetClocksFreq(&RCC_Clocks);

	//ustawienie przerwania SysTick
	SysTick_Config(RCC_Clocks.SYSCLK_Frequency / 5);

}

void Configuration_ADC(void)
{
	ADC_InitTypeDef ADC_InitStructure;

	//konfiguracja ADC1
	ADC_InitStructure.ADC_Mode = ADC_Mode_Independent;
	ADC_InitStructure.ADC_ScanConvMode = DISABLE;
	ADC_InitStructure.ADC_ContinuousConvMode = DISABLE;
	ADC_InitStructure.ADC_ExternalTrigConv = ADC_ExternalTrigConv_None;
	ADC_InitStructure.ADC_DataAlign = ADC_DataAlign_Right;
	ADC_InitStructure.ADC_NbrOfChannel = 1;
	ADC_Init(ADC1, &ADC_InitStructure);

	//
	ADC_RegularChannelConfig(ADC1, ADC_Channel_14, 1, ADC_SampleTime_71Cycles5);

	//wczenie ADC1
	ADC_Cmd(ADC1, ENABLE);

	//wywoanie resetu rejestrw kalibracji
	ADC_ResetCalibration(ADC1);
	//oczekiwanie na zakoczenie resetu
	while(ADC_GetResetCalibrationStatus(ADC1));

	//wywoanie kalibracji
	ADC_StartCalibration(ADC1);
	//oczekiwanie na zakoczenie kalibracji
	while(ADC_GetCalibrationStatus(ADC1));

}

#ifdef  USE_FULL_ASSERT

/**
  * @brief  Reports the name of the source file and the source line number
  *   where the assert_param error has occurred.
  * @param  file: pointer to the source file name
  * @param  line: assert_param error line source number
  * @retval None
  */
void assert_failed(uint8_t* file, uint32_t line)
{
  /* User can add his own implementation to report the file name and line number,
     ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

  /* Infinite loop */
  while (1)
  {
  }
}
#endif

/*
 * Minimal __assert_func used by the assert() macro
 * */
void __assert_func(const char *file, int line, const char *func, const char *failedexpr)
{
  while(1)
  {}
}

/*
 * Minimal __assert() uses __assert__func()
 * */
void __assert(const char *file, int line, const char *failedexpr)
{
   __assert_func (file, line, NULL, failedexpr);
}

#ifdef USE_SEE
#ifndef USE_DEFAULT_TIMEOUT_CALLBACK
/**
  * @brief  Basic management of the timeout situation.
  * @param  None.
  * @retval sEE_FAIL.
  */
uint32_t sEE_TIMEOUT_UserCallback(void)
{
  /* Return with error code */
  return sEE_FAIL;
}
#endif
#endif /* USE_SEE */

